@@ -150,6 +150,18 @@ class _PrefixedArgsIterator:
         return self.index >= self.length
 
 
+def _check_for_no_arg(
+    anno: typing.Any, type_to_converter: dict
+) -> typing_extensions.TypeGuard[converters.NoArgumentConverter]:
+    return (
+        isinstance(anno, converters.NoArgumentConverter)
+        or (inspect.isclass(anno) and issubclass(anno, converters.NoArgumentConverter))
+        or issubclass(
+            type_to_converter.get(anno, object), converters.NoArgumentConverter
+        )
+    )
+
+
 def _get_name(x: typing.Any):
     try:
         return x.__name__
@@ -232,16 +244,20 @@ def _get_converter(
         literals = typing_extensions.get_args(anno)
         return converters._LiteralConverter(literals).convert
 
-    elif inspect.isfunction(anno):
+    elif inspect.isroutine(anno):
         num_params = len(inspect.signature(anno).parameters.values())
         if num_params == 0:
             ValueError(
                 f"{_get_name(anno)} for {name} has 0 arguments, which is unsupported."
             )
         elif num_params == 1:
-            return lambda ctx, arg: anno(arg)
+
+            async def _one_function_cmd(ctx, arg) -> typing.Any:
+                return await maybe_coroutine(anno, arg)
+
+            return _one_function_cmd
         elif num_params == 2:
-            return lambda ctx, arg: anno(ctx, arg)
+            return anno
         else:
             ValueError(
                 f"{_get_name(anno)} for {name} has more than 2 arguments, which is"
@@ -326,9 +342,7 @@ def _get_params(
         if typing_extensions.get_origin(anno) in UNION_TYPES:
             cmd_param.union = True
             for arg in typing_extensions.get_args(anno):
-                if isinstance(anno, converters.NoArgumentConverter) or issubclass(
-                    type_to_converter.get(anno, object), converters.NoArgumentConverter
-                ):
+                if _check_for_no_arg(anno, type_to_converter):
                     cmd_param.no_argument = True
 
                 if arg != NoneType:
@@ -337,9 +351,7 @@ def _get_params(
                 elif not cmd_param.optional:  # d.py-like behavior
                     cmd_param.default = None
         else:
-            if isinstance(anno, converters.NoArgumentConverter) or issubclass(
-                type_to_converter.get(anno, object), converters.NoArgumentConverter
-            ):
+            if _check_for_no_arg(anno, type_to_converter):
                 cmd_param.no_argument = True
 
             converter = _get_converter(anno, name, type_to_converter)  # type: ignore
@@ -883,6 +895,8 @@ class MolterCommand:
         callback = self.callback
 
         if len(self.parameters) == 0:
+            if ctx.args and not self.ignore_extra:
+                raise errors.BadArgument(f"Too many arguments passed to {self.name}.")
             return await callback(ctx)
         else:
             new_args: list[typing.Any] = []
